<?php

namespace Project\Role\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Project\Role\Http\Requests\RoleRequest;
use Project\Role\Interfaces\RoleRepositoryInterface;
use Project\Role\Models\Role;

/**
 * Resource controller class for role.
 */
class RoleResourceController extends BaseController
{

    /**
     * Initialize role resource controller.
     *
     * @param type RoleRepositoryInterface $role
     *
     * @return null
     */
    public function __construct(RoleRepositoryInterface $role)
    {
        parent::__construct();
        $this->repository = $role;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Project\Role\Repositories\Criteria\RoleResourceCriteria::class);
    }

    /**
     * Display a list of role.
     *
     * @return Response
     */
    public function index(RoleRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Project\Role\Repositories\Presenter\RolePresenter::class)
                ->$function();
        }

        $roles = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('role::role.names'))
            ->view('role::role.index', true)
            ->data(compact('roles', 'view'))
            ->output();
    }

    /**
     * Display role.
     *
     * @param Request $request
     * @param Model   $role
     *
     * @return Response
     */
    public function show(RoleRequest $request, Role $role)
    {

        if ($role->exists) {
            $view = 'role::role.show';
        } else {
            $view = 'role::role.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('role::role.name'))
            ->data(compact('role'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new role.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(RoleRequest $request)
    {

        $role = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('role::role.name')) 
            ->view('role::role.create', true) 
            ->data(compact('role'))
            ->output();
    }

    /**
     * Create new role.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(RoleRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $role                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('role::role.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('role/role/' . $role->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/role/role'))
                ->redirect();
        }

    }

    /**
     * Show role for editing.
     *
     * @param Request $request
     * @param Model   $role
     *
     * @return Response
     */
    public function edit(RoleRequest $request, Role $role)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('role::role.name'))
            ->view('role::role.edit', true)
            ->data(compact('role'))
            ->output();
    }

    /**
     * Update the role.
     *
     * @param Request $request
     * @param Model   $role
     *
     * @return Response
     */
    public function update(RoleRequest $request, Role $role)
    {
        try {
            $attributes = $request->all();

            $role->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('role::role.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('role/role/' . $role->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('role/role/' . $role->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the role.
     *
     * @param Model   $role
     *
     * @return Response
     */
    public function destroy(RoleRequest $request, Role $role)
    {
        try {

            $role->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('role::role.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('role/role/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('role/role/' . $role->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple role.
     *
     * @param Model   $role
     *
     * @return Response
     */
    public function delete(RoleRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('role::role.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('role/role'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/role/role'))
                ->redirect();
        }

    }

    /**
     * Restore deleted roles.
     *
     * @param Model   $role
     *
     * @return Response
     */
    public function restore(RoleRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('role::role.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/role/role'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/role/role/'))
                ->redirect();
        }

    }

}
